@@ -1,46 +1,62 @@
-# pylint: disable=E1103
-
-
-from pandas.core.dtypes.common import (
-    is_list_like, is_scalar, is_integer_dtype)
-from pandas.core.dtypes.generic import ABCDataFrame, ABCSeries
-from pandas.core.dtypes.cast import maybe_downcast_to_dtype
-
-from pandas.core.reshape.concat import concat
-from pandas.core.series import Series
-from pandas.core.groupby.groupby import Grouper
-from pandas.core.reshape.util import cartesian_product
-from pandas.core.index import Index, _get_objs_combined_axis
-from pandas.compat import range, lrange, zip
-from pandas import compat
-import pandas.core.common as com
-from pandas.util._decorators import Appender, Substitution
-
-from pandas.core.frame import _shared_docs
-# Note: We need to make sure `frame` is imported before `pivot`, otherwise
-# _shared_docs['pivot_table'] will not yet exist.  TODO: Fix this dependency
+from typing import TYPE_CHECKING, Callable, Dict, List, Tuple, Union
 
 import numpy as np
 
+from pandas.util._decorators import Appender, Substitution
 
-@Substitution('\ndata : DataFrame')
-@Appender(_shared_docs['pivot_table'], indents=1)
-def pivot_table(data, values=None, index=None, columns=None, aggfunc='mean',
-                fill_value=None, margins=False, dropna=True,
-                margins_name='All'):
+from pandas.core.dtypes.cast import maybe_downcast_to_dtype
+from pandas.core.dtypes.common import is_integer_dtype, is_list_like, is_scalar
+from pandas.core.dtypes.generic import ABCDataFrame, ABCSeries
+
+import pandas.core.common as com
+from pandas.core.frame import _shared_docs
+from pandas.core.groupby import Grouper
+from pandas.core.indexes.api import Index, MultiIndex, get_objs_combined_axis
+from pandas.core.reshape.concat import concat
+from pandas.core.reshape.util import cartesian_product
+from pandas.core.series import Series
+
+if TYPE_CHECKING:
+    from pandas import DataFrame
+
+
+# Note: We need to make sure `frame` is imported before `pivot`, otherwise
+# _shared_docs['pivot_table'] will not yet exist.  TODO: Fix this dependency
+@Substitution("\ndata : DataFrame")
+@Appender(_shared_docs["pivot_table"], indents=1)
+def pivot_table(
+    data,
+    values=None,
+    index=None,
+    columns=None,
+    aggfunc="mean",
+    fill_value=None,
+    margins=False,
+    dropna=True,
+    margins_name="All",
+    observed=False,
+) -> "DataFrame":
     index = _convert_by(index)
     columns = _convert_by(columns)
 
     if isinstance(aggfunc, list):
-        pieces = []
+        pieces: List[DataFrame] = []
         keys = []
         for func in aggfunc:
-            table = pivot_table(data, values=values, index=index,
-                                columns=columns,
-                                fill_value=fill_value, aggfunc=func,
-                                margins=margins, margins_name=margins_name)
+            table = pivot_table(
+                data,
+                values=values,
+                index=index,
+                columns=columns,
+                fill_value=fill_value,
+                aggfunc=func,
+                margins=margins,
+                dropna=dropna,
+                margins_name=margins_name,
+                observed=observed,
+            )
             pieces.append(table)
-            keys.append(getattr(func, '__name__', func))
+            keys.append(getattr(func, "__name__", func))
 
         return concat(pieces, keys=keys, axis=1)
 
@@ -81,21 +97,23 @@ def pivot_table(data, values=None, index=None, columns=None, aggfunc='mean',
                 pass
         values = list(values)
 
-    # group by the cartesian product of the grouper
-    # if we have a categorical
-    grouped = data.groupby(keys, observed=False)
+    grouped = data.groupby(keys, observed=observed)
     agged = grouped.agg(aggfunc)
     if dropna and isinstance(agged, ABCDataFrame) and len(agged.columns):
-        agged = agged.dropna(how='all')
+        agged = agged.dropna(how="all")
 
         # gh-21133
         # we want to down cast if
         # the original values are ints
         # as we grouped with a NaN value
         # and then dropped, coercing to floats
-        for v in [v for v in values if v in data and v in agged]:
-            if (is_integer_dtype(data[v]) and
-                    not is_integer_dtype(agged[v])):
+        for v in values:
+            if (
+                v in data
+                and is_integer_dtype(data[v])
+                and v in agged
+                and not is_integer_dtype(agged[v])
+            ):
                 agged[v] = maybe_downcast_to_dtype(agged[v], data[v].dtype)
 
     table = agged
@@ -103,7 +121,7 @@ def pivot_table(data, values=None, index=None, columns=None, aggfunc='mean',
         # Related GH #17123
         # If index_names are integers, determine whether the integers refer
         # to the level position or name.
-        index_names = agged.index.names[:len(index)]
+        index_names = agged.index.names[: len(index)]
         to_unstack = []
         for i in range(len(index), len(keys)):
             name = agged.index.names[i]
@@ -114,34 +132,48 @@ def pivot_table(data, values=None, index=None, columns=None, aggfunc='mean',
         table = agged.unstack(to_unstack)
 
     if not dropna:
-        from pandas import MultiIndex
         if table.index.nlevels > 1:
-            m = MultiIndex.from_arrays(cartesian_product(table.index.levels),
-                                       names=table.index.names)
+            m = MultiIndex.from_arrays(
+                cartesian_product(table.index.levels), names=table.index.names
+            )
             table = table.reindex(m, axis=0)
 
         if table.columns.nlevels > 1:
-            m = MultiIndex.from_arrays(cartesian_product(table.columns.levels),
-                                       names=table.columns.names)
+            m = MultiIndex.from_arrays(
+                cartesian_product(table.columns.levels), names=table.columns.names
+            )
             table = table.reindex(m, axis=1)
 
     if isinstance(table, ABCDataFrame):
         table = table.sort_index(axis=1)
 
     if fill_value is not None:
-        table = table.fillna(value=fill_value, downcast='infer')
+        _table = table.fillna(fill_value, downcast="infer")
+        assert _table is not None  # needed for mypy
+        table = _table
 
     if margins:
         if dropna:
             data = data[data.notna().all(axis=1)]
-        table = _add_margins(table, data, values, rows=index,
-                             cols=columns, aggfunc=aggfunc,
-                             observed=dropna,
-                             margins_name=margins_name, fill_value=fill_value)
+        table = _add_margins(
+            table,
+            data,
+            values,
+            rows=index,
+            cols=columns,
+            aggfunc=aggfunc,
+            observed=dropna,
+            margins_name=margins_name,
+            fill_value=fill_value,
+        )
 
     # discard the top level
-    if values_passed and not values_multi and not table.empty and \
-       (table.columns.nlevels > 1):
+    if (
+        values_passed
+        and not values_multi
+        and not table.empty
+        and (table.columns.nlevels > 1)
+    ):
         table = table[values[0]]
 
     if len(index) == 0 and len(columns) > 0:
@@ -149,31 +181,41 @@ def pivot_table(data, values=None, index=None, columns=None, aggfunc='mean',
 
     # GH 15193 Make sure empty columns are removed if dropna=True
     if isinstance(table, ABCDataFrame) and dropna:
-        table = table.dropna(how='all', axis=1)
+        table = table.dropna(how="all", axis=1)
 
     return table
 
 
-def _add_margins(table, data, values, rows, cols, aggfunc,
-                 observed=None, margins_name='All', fill_value=None):
-    if not isinstance(margins_name, compat.string_types):
-        raise ValueError('margins_name argument must be a string')
+def _add_margins(
+    table: Union["Series", "DataFrame"],
+    data,
+    values,
+    rows,
+    cols,
+    aggfunc,
+    observed=None,
+    margins_name: str = "All",
+    fill_value=None,
+):
+    if not isinstance(margins_name, str):
+        raise ValueError("margins_name argument must be a string")
 
-    msg = u'Conflicting name "{name}" in margins'.format(name=margins_name)
+    msg = 'Conflicting name "{name}" in margins'.format(name=margins_name)
     for level in table.index.names:
         if margins_name in table.index.get_level_values(level):
             raise ValueError(msg)
 
     grand_margin = _compute_grand_margin(data, values, aggfunc, margins_name)
 
-    # could be passed a Series object with no 'columns'
-    if hasattr(table, 'columns'):
+    if table.ndim == 2:
+        # i.e. DataFramae
         for level in table.columns.names[1:]:
             if margins_name in table.columns.get_level_values(level):
                 raise ValueError(msg)
 
+    key: Union[str, Tuple[str, ...]]
     if len(rows) > 1:
-        key = (margins_name,) + ('',) * (len(rows) - 1)
+        key = (margins_name,) + ("",) * (len(rows) - 1)
     else:
         key = margins_name
 
@@ -182,37 +224,51 @@ def _add_margins(table, data, values, rows, cols, aggfunc,
         # one column in the data. Compute grand margin and return it.
         return table.append(Series({key: grand_margin[margins_name]}))
 
-    if values:
-        marginal_result_set = _generate_marginal_results(table, data, values,
-                                                         rows, cols, aggfunc,
-                                                         observed,
-                                                         grand_margin,
-                                                         margins_name)
+    elif values:
+        marginal_result_set = _generate_marginal_results(
+            table,
+            data,
+            values,
+            rows,
+            cols,
+            aggfunc,
+            observed,
+            grand_margin,
+            margins_name,
+        )
         if not isinstance(marginal_result_set, tuple):
             return marginal_result_set
         result, margin_keys, row_margin = marginal_result_set
     else:
+        # no values, and table is a DataFrame
+        assert isinstance(table, ABCDataFrame)
         marginal_result_set = _generate_marginal_results_without_values(
-            table, data, rows, cols, aggfunc, observed, margins_name)
+            table, data, rows, cols, aggfunc, observed, margins_name
+        )
         if not isinstance(marginal_result_set, tuple):
             return marginal_result_set
         result, margin_keys, row_margin = marginal_result_set
+
     row_margin = row_margin.reindex(result.columns, fill_value=fill_value)
     # populate grand margin
     for k in margin_keys:
-        if isinstance(k, compat.string_types):
+        if isinstance(k, str):
             row_margin[k] = grand_margin[k]
         else:
             row_margin[k] = grand_margin[k[0]]
 
     from pandas import DataFrame
+
     margin_dummy = DataFrame(row_margin, columns=[key]).T
 
     row_names = result.index.names
     try:
+        # check the result column and leave floats
         for dtype in set(result.dtypes):
             cols = result.select_dtypes([dtype]).columns
-            margin_dummy[cols] = margin_dummy[cols].astype(dtype)
+            margin_dummy[cols] = margin_dummy[cols].apply(
+                maybe_downcast_to_dtype, args=(dtype,)
+            )
         result = result.append(margin_dummy)
     except TypeError:
 
@@ -224,17 +280,16 @@ def _add_margins(table, data, values, rows, cols, aggfunc,
     return result
 
 
-def _compute_grand_margin(data, values, aggfunc,
-                          margins_name='All'):
+def _compute_grand_margin(data, values, aggfunc, margins_name: str = "All"):
 
     if values:
         grand_margin = {}
-        for k, v in data[values].iteritems():
+        for k, v in data[values].items():
             try:
-                if isinstance(aggfunc, compat.string_types):
+                if isinstance(aggfunc, str):
                     grand_margin[k] = getattr(v, aggfunc)()
                 elif isinstance(aggfunc, dict):
-                    if isinstance(aggfunc[k], compat.string_types):
+                    if isinstance(aggfunc[k], str):
                         grand_margin[k] = getattr(v, aggfunc[k])()
                     else:
                         grand_margin[k] = aggfunc[k](v)
@@ -247,26 +302,30 @@ def _compute_grand_margin(data, values, aggfunc,
         return {margins_name: aggfunc(data.index)}
 
 
-def _generate_marginal_results(table, data, values, rows, cols, aggfunc,
-                               observed,
-                               grand_margin,
-                               margins_name='All'):
+def _generate_marginal_results(
+    table,
+    data,
+    values,
+    rows,
+    cols,
+    aggfunc,
+    observed,
+    grand_margin,
+    margins_name: str = "All",
+):
     if len(cols) > 0:
         # need to "interleave" the margins
         table_pieces = []
         margin_keys = []
 
         def _all_key(key):
-            return (key, margins_name) + ('',) * (len(cols) - 1)
+            return (key, margins_name) + ("",) * (len(cols) - 1)
 
         if len(rows) > 0:
-            margin = data[rows + values].groupby(
-                rows, observed=observed).agg(aggfunc)
+            margin = data[rows + values].groupby(rows, observed=observed).agg(aggfunc)
             cat_axis = 1
 
-            for key, piece in table.groupby(level=0,
-                                            axis=cat_axis,
-                                            observed=observed):
+            for key, piece in table.groupby(level=0, axis=cat_axis, observed=observed):
                 all_key = _all_key(key)
 
                 # we are going to mutate this, so need to copy!
@@ -276,9 +335,11 @@ def _generate_marginal_results(table, data, values, rows, cols, aggfunc,
                 except TypeError:
 
                     # we cannot reshape, so coerce the axis
-                    piece.set_axis(piece._get_axis(
-                                   cat_axis)._to_safe_for_reshape(),
-                                   axis=cat_axis, inplace=True)
+                    piece.set_axis(
+                        piece._get_axis(cat_axis)._to_safe_for_reshape(),
+                        axis=cat_axis,
+                        inplace=True,
+                    )
                     piece[all_key] = margin[key]
 
                 table_pieces.append(piece)
@@ -286,9 +347,7 @@ def _generate_marginal_results(table, data, values, rows, cols, aggfunc,
         else:
             margin = grand_margin
             cat_axis = 0
-            for key, piece in table.groupby(level=0,
-                                            axis=cat_axis,
-                                            observed=observed):
+            for key, piece in table.groupby(level=0, axis=cat_axis, observed=observed):
                 all_key = _all_key(key)
                 table_pieces.append(piece)
                 table_pieces.append(Series(margin[key], index=[all_key]))
@@ -303,12 +362,11 @@ def _generate_marginal_results(table, data, values, rows, cols, aggfunc,
         margin_keys = table.columns
 
     if len(cols) > 0:
-        row_margin = data[cols + values].groupby(
-            cols, observed=observed).agg(aggfunc)
+        row_margin = data[cols + values].groupby(cols, observed=observed).agg(aggfunc)
         row_margin = row_margin.stack()
 
         # slight hack
-        new_order = [len(cols)] + lrange(len(cols))
+        new_order = [len(cols)] + list(range(len(cols)))
         row_margin.index = row_margin.index.reorder_levels(new_order)
     else:
         row_margin = Series(np.nan, index=result.columns)
@@ -317,8 +375,8 @@ def _generate_marginal_results(table, data, values, rows, cols, aggfunc,
 
 
 def _generate_marginal_results_without_values(
-        table, data, rows, cols, aggfunc,
-        observed, margins_name='All'):
+    table: "DataFrame", data, rows, cols, aggfunc, observed, margins_name: str = "All"
+):
     if len(cols) > 0:
         # need to "interleave" the margins
         margin_keys = []
@@ -326,20 +384,17 @@ def _generate_marginal_results_without_values(
         def _all_key():
             if len(cols) == 1:
                 return margins_name
-            return (margins_name, ) + ('', ) * (len(cols) - 1)
+            return (margins_name,) + ("",) * (len(cols) - 1)
 
         if len(rows) > 0:
-            margin = data[rows].groupby(rows,
-                                        observed=observed).apply(aggfunc)
+            margin = data[rows].groupby(rows, observed=observed).apply(aggfunc)
             all_key = _all_key()
             table[all_key] = margin
             result = table
             margin_keys.append(all_key)
 
         else:
-            margin = data.groupby(level=0,
-                                  axis=0,
-                                  observed=observed).apply(aggfunc)
+            margin = data.groupby(level=0, axis=0, observed=observed).apply(aggfunc)
             all_key = _all_key()
             table[all_key] = margin
             result = table
@@ -360,49 +415,84 @@ def _generate_marginal_results_without_values(
 def _convert_by(by):
     if by is None:
         by = []
-    elif (is_scalar(by) or
-          isinstance(by, (np.ndarray, Index, ABCSeries, Grouper)) or
-          hasattr(by, '__call__')):
+    elif (
+        is_scalar(by)
+        or isinstance(by, (np.ndarray, Index, ABCSeries, Grouper))
+        or hasattr(by, "__call__")
+    ):
         by = [by]
     else:
         by = list(by)
     return by
 
 
-def crosstab(index, columns, values=None, rownames=None, colnames=None,
-             aggfunc=None, margins=False, margins_name='All', dropna=True,
-             normalize=False):
+@Substitution("\ndata : DataFrame")
+@Appender(_shared_docs["pivot"], indents=1)
+def pivot(data: "DataFrame", index=None, columns=None, values=None) -> "DataFrame":
+    if values is None:
+        cols = [columns] if index is None else [index, columns]
+        append = index is None
+        indexed = data.set_index(cols, append=append)
+    else:
+        if index is None:
+            index = data.index
+        else:
+            index = data[index]
+        index = MultiIndex.from_arrays([index, data[columns]])
+
+        if is_list_like(values) and not isinstance(values, tuple):
+            # Exclude tuple because it is seen as a single column name
+            indexed = data._constructor(
+                data[values].values, index=index, columns=values
+            )
+        else:
+            indexed = data._constructor_sliced(data[values].values, index=index)
+    return indexed.unstack(columns)
+
+
+def crosstab(
+    index,
+    columns,
+    values=None,
+    rownames=None,
+    colnames=None,
+    aggfunc=None,
+    margins=False,
+    margins_name: str = "All",
+    dropna: bool = True,
+    normalize=False,
+) -> "DataFrame":
     """
-    Compute a simple cross-tabulation of two (or more) factors. By default
+    Compute a simple cross tabulation of two (or more) factors. By default
     computes a frequency table of the factors unless an array of values and an
-    aggregation function are passed
+    aggregation function are passed.
 
     Parameters
     ----------
     index : array-like, Series, or list of arrays/Series
-        Values to group by in the rows
+        Values to group by in the rows.
     columns : array-like, Series, or list of arrays/Series
-        Values to group by in the columns
+        Values to group by in the columns.
     values : array-like, optional
         Array of values to aggregate according to the factors.
         Requires `aggfunc` be specified.
-    aggfunc : function, optional
-        If specified, requires `values` be specified as well
     rownames : sequence, default None
-        If passed, must match number of row arrays passed
+        If passed, must match number of row arrays passed.
     colnames : sequence, default None
-        If passed, must match number of column arrays passed
-    margins : boolean, default False
-        Add row/column margins (subtotals)
-    margins_name : string, default 'All'
-        Name of the row / column that will contain the totals
+        If passed, must match number of column arrays passed.
+    aggfunc : function, optional
+        If specified, requires `values` be specified as well.
+    margins : bool, default False
+        Add row/column margins (subtotals).
+    margins_name : str, default 'All'
+        Name of the row/column that will contain the totals
         when margins is True.
 
         .. versionadded:: 0.21.0
 
-    dropna : boolean, default True
-        Do not include columns whose entries are all NaN
-    normalize : boolean, {'all', 'index', 'columns'}, or {0,1}, default False
+    dropna : bool, default True
+        Do not include columns whose entries are all NaN.
+    normalize : bool, {'all', 'index', 'columns'}, or {0,1}, default False
         Normalize by dividing all values by the sum of values.
 
         - If passed 'all' or `True`, will normalize over all values.
@@ -410,8 +500,15 @@ def crosstab(index, columns, values=None, rownames=None, colnames=None,
         - If passed 'columns' will normalize over each column.
         - If margins is `True`, will also normalize margin values.
 
-        .. versionadded:: 0.18.1
+    Returns
+    -------
+    DataFrame
+        Cross tabulation of the data.
 
+    See Also
+    --------
+    DataFrame.pivot : Reshape data based on column values.
+    pivot_table : Create a pivot table as a DataFrame.
 
     Notes
     -----
@@ -433,42 +530,45 @@ def crosstab(index, columns, values=None, rownames=None, colnames=None,
     ...               "one", "two", "two", "two", "one"], dtype=object)
     >>> c = np.array(["dull", "dull", "shiny", "dull", "dull", "shiny",
     ...               "shiny", "dull", "shiny", "shiny", "shiny"],
-    ...               dtype=object)
-
+    ...              dtype=object)
     >>> pd.crosstab(a, [b, c], rownames=['a'], colnames=['b', 'c'])
-    ... # doctest: +NORMALIZE_WHITESPACE
     b   one        two
     c   dull shiny dull shiny
     a
     bar    1     2    1     0
     foo    2     2    1     2
 
+    Here 'c' and 'f' are not represented in the data and will not be
+    shown in the output because dropna is True by default. Set
+    dropna=False to preserve categories with no data.
+
     >>> foo = pd.Categorical(['a', 'b'], categories=['a', 'b', 'c'])
     >>> bar = pd.Categorical(['d', 'e'], categories=['d', 'e', 'f'])
-    >>> crosstab(foo, bar)  # 'c' and 'f' are not represented in the data,
-    ...                     # but they still will be counted in the output
-    ... # doctest: +SKIP
+    >>> pd.crosstab(foo, bar)
+    col_0  d  e
+    row_0
+    a      1  0
+    b      0  1
+    >>> pd.crosstab(foo, bar, dropna=False)
     col_0  d  e  f
     row_0
     a      1  0  0
     b      0  1  0
     c      0  0  0
-
-    Returns
-    -------
-    crosstab : DataFrame
     """
 
-    index = com._maybe_make_list(index)
-    columns = com._maybe_make_list(columns)
+    index = com.maybe_make_list(index)
+    columns = com.maybe_make_list(columns)
 
-    rownames = _get_names(index, rownames, prefix='row')
-    colnames = _get_names(columns, colnames, prefix='col')
+    rownames = _get_names(index, rownames, prefix="row")
+    colnames = _get_names(columns, colnames, prefix="col")
 
-    common_idx = _get_objs_combined_axis(index + columns, intersect=True,
-                                         sort=False)
+    common_idx = None
+    pass_objs = [x for x in index + columns if isinstance(x, (ABCSeries, ABCDataFrame))]
+    if pass_objs:
+        common_idx = get_objs_combined_axis(pass_objs, intersect=True, sort=False)
 
-    data = {}
+    data: Dict = {}
     data.update(zip(rownames, index))
     data.update(zip(colnames, columns))
 
@@ -479,31 +579,38 @@ def crosstab(index, columns, values=None, rownames=None, colnames=None,
         raise ValueError("values cannot be used without an aggfunc.")
 
     from pandas import DataFrame
+
     df = DataFrame(data, index=common_idx)
     if values is None:
-        df['__dummy__'] = 0
-        kwargs = {'aggfunc': len, 'fill_value': 0}
+        df["__dummy__"] = 0
+        kwargs = {"aggfunc": len, "fill_value": 0}
     else:
-        df['__dummy__'] = values
-        kwargs = {'aggfunc': aggfunc}
+        df["__dummy__"] = values
+        kwargs = {"aggfunc": aggfunc}
 
-    table = df.pivot_table('__dummy__', index=rownames, columns=colnames,
-                           margins=margins, margins_name=margins_name,
-                           dropna=dropna, **kwargs)
+    table = df.pivot_table(
+        "__dummy__",
+        index=rownames,
+        columns=colnames,
+        margins=margins,
+        margins_name=margins_name,
+        dropna=dropna,
+        **kwargs,
+    )
 
     # Post-process
     if normalize is not False:
-        table = _normalize(table, normalize=normalize, margins=margins,
-                           margins_name=margins_name)
+        table = _normalize(
+            table, normalize=normalize, margins=margins, margins_name=margins_name
+        )
 
     return table
 
 
-def _normalize(table, normalize, margins, margins_name='All'):
+def _normalize(table, normalize, margins: bool, margins_name="All"):
 
-    if not isinstance(normalize, bool) and not isinstance(normalize,
-                                                          compat.string_types):
-        axis_subs = {0: 'index', 1: 'columns'}
+    if not isinstance(normalize, (bool, str)):
+        axis_subs = {0: "index", 1: "columns"}
         try:
             normalize = axis_subs[normalize]
         except KeyError:
@@ -512,13 +619,13 @@ def _normalize(table, normalize, margins, margins_name='All'):
     if margins is False:
 
         # Actual Normalizations
-        normalizers = {
-            'all': lambda x: x / x.sum(axis=1).sum(axis=0),
-            'columns': lambda x: x / x.sum(),
-            'index': lambda x: x.div(x.sum(axis=1), axis=0)
+        normalizers: Dict[Union[bool, str], Callable] = {
+            "all": lambda x: x / x.sum(axis=1).sum(axis=0),
+            "columns": lambda x: x / x.sum(),
+            "index": lambda x: x.div(x.sum(axis=1), axis=0),
         }
 
-        normalizers[True] = normalizers['all']
+        normalizers[True] = normalizers["all"]
 
         try:
             f = normalizers[normalize]
@@ -529,27 +636,39 @@ def _normalize(table, normalize, margins, margins_name='All'):
         table = table.fillna(0)
 
     elif margins is True:
+        # keep index and column of pivoted table
+        table_index = table.index
+        table_columns = table.columns
 
-        column_margin = table.loc[:, margins_name].drop(margins_name)
-        index_margin = table.loc[margins_name, :].drop(margins_name)
-        table = table.drop(margins_name, axis=1).drop(margins_name)
-        # to keep index and columns names
-        table_index_names = table.index.names
-        table_columns_names = table.columns.names
+        # check if margin name is in (for MI cases) or equal to last
+        # index/column and save the column and index margin
+        if (margins_name not in table.iloc[-1, :].name) | (
+            margins_name != table.iloc[:, -1].name
+        ):
+            raise ValueError(
+                "{mname} not in pivoted DataFrame".format(mname=margins_name)
+            )
+        column_margin = table.iloc[:-1, -1]
+        index_margin = table.iloc[-1, :-1]
+
+        # keep the core table
+        table = table.iloc[:-1, :-1]
 
         # Normalize core
         table = _normalize(table, normalize=normalize, margins=False)
 
         # Fix Margins
-        if normalize == 'columns':
+        if normalize == "columns":
             column_margin = column_margin / column_margin.sum()
             table = concat([table, column_margin], axis=1)
             table = table.fillna(0)
+            table.columns = table_columns
 
-        elif normalize == 'index':
+        elif normalize == "index":
             index_margin = index_margin / index_margin.sum()
             table = table.append(index_margin)
             table = table.fillna(0)
+            table.index = table_index
 
         elif normalize == "all" or normalize is True:
             column_margin = column_margin / column_margin.sum()
@@ -559,30 +678,29 @@ def _normalize(table, normalize, margins, margins_name='All'):
             table = table.append(index_margin)
 
             table = table.fillna(0)
+            table.index = table_index
+            table.columns = table_columns
 
         else:
             raise ValueError("Not a valid normalize argument")
 
-        table.index.names = table_index_names
-        table.columns.names = table_columns_names
-
     else:
         raise ValueError("Not a valid margins argument")
 
     return table
 
 
-def _get_names(arrs, names, prefix='row'):
+def _get_names(arrs, names, prefix: str = "row"):
     if names is None:
         names = []
         for i, arr in enumerate(arrs):
             if isinstance(arr, ABCSeries) and arr.name is not None:
                 names.append(arr.name)
             else:
-                names.append('{prefix}_{i}'.format(prefix=prefix, i=i))
+                names.append("{prefix}_{i}".format(prefix=prefix, i=i))
     else:
         if len(names) != len(arrs):
-            raise AssertionError('arrays and names must have the same length')
+            raise AssertionError("arrays and names must have the same length")
         if not isinstance(names, list):
             names = list(names)
 
