@@ -1,14 +1,14 @@
 """Utilities for conversion to writer-agnostic Excel representation
 """
 
+from functools import reduce
 import itertools
 import re
+from typing import Callable, Dict, List, Optional, Sequence, Union
 import warnings
 
 import numpy as np
 
-from pandas.compat import reduce
-
 from pandas.core.dtypes import missing
 from pandas.core.dtypes.common import is_float, is_scalar
 from pandas.core.dtypes.generic import ABCMultiIndex, ABCPeriodIndex
@@ -16,17 +16,19 @@ from pandas.core.dtypes.generic import ABCMultiIndex, ABCPeriodIndex
 from pandas import Index
 import pandas.core.common as com
 
+from pandas.io.common import stringify_path
 from pandas.io.formats.css import CSSResolver, CSSWarning
 from pandas.io.formats.format import get_level_lengths
 from pandas.io.formats.printing import pprint_thing
 
 
-class ExcelCell(object):
-    __fields__ = ('row', 'col', 'val', 'style', 'mergestart', 'mergeend')
+class ExcelCell:
+    __fields__ = ("row", "col", "val", "style", "mergestart", "mergeend")
     __slots__ = __fields__
 
-    def __init__(self, row, col, val, style=None, mergestart=None,
-                 mergeend=None):
+    def __init__(
+        self, row: int, col: int, val, style=None, mergestart=None, mergeend=None
+    ):
         self.row = row
         self.col = col
         self.val = val
@@ -35,7 +37,7 @@ class ExcelCell(object):
         self.mergeend = mergeend
 
 
-class CSSToExcelConverter(object):
+class CSSToExcelConverter:
     """A callable for converting CSS declarations to ExcelWriter styles
 
     Supports parts of CSS 2.2, with minimal CSS 3.0 support (e.g. text-shadow),
@@ -51,22 +53,23 @@ class CSSToExcelConverter(object):
         CSS declarations understood to be the containing scope for the
         CSS processed by :meth:`__call__`.
     """
+
     # NB: Most of the methods here could be classmethods, as only __init__
     #     and __call__ make use of instance attributes.  We leave them as
     #     instancemethods so that users can easily experiment with extensions
     #     without monkey-patching.
 
-    def __init__(self, inherited=None):
+    def __init__(self, inherited: Optional[str] = None):
         if inherited is not None:
-            inherited = self.compute_css(inherited,
-                                         self.compute_css.INITIAL_STYLE)
+            inherited = self.compute_css(inherited)
 
         self.inherited = inherited
 
     compute_css = CSSResolver()
 
-    def __call__(self, declarations_str):
-        """Convert CSS declarations to ExcelWriter style
+    def __call__(self, declarations_str: str) -> Dict[str, Dict[str, str]]:
+        """
+        Convert CSS declarations to ExcelWriter style.
 
         Parameters
         ----------
@@ -84,17 +87,18 @@ class CSSToExcelConverter(object):
         properties = self.compute_css(declarations_str, self.inherited)
         return self.build_xlstyle(properties)
 
-    def build_xlstyle(self, props):
+    def build_xlstyle(self, props: Dict[str, str]) -> Dict[str, Dict[str, str]]:
         out = {
-            'alignment': self.build_alignment(props),
-            'border': self.build_border(props),
-            'fill': self.build_fill(props),
-            'font': self.build_font(props),
-            'number_format': self.build_number_format(props),
+            "alignment": self.build_alignment(props),
+            "border": self.build_border(props),
+            "fill": self.build_fill(props),
+            "font": self.build_font(props),
+            "number_format": self.build_number_format(props),
         }
+
         # TODO: handle cell width and height: needs support in pandas.io.excel
 
-        def remove_none(d):
+        def remove_none(d: Dict[str, str]) -> None:
             """Remove key where value is None, through nested dicts"""
             for k, v in list(d.items()):
                 if v is None:
@@ -108,35 +112,40 @@ class CSSToExcelConverter(object):
         return out
 
     VERTICAL_MAP = {
-        'top': 'top',
-        'text-top': 'top',
-        'middle': 'center',
-        'baseline': 'bottom',
-        'bottom': 'bottom',
-        'text-bottom': 'bottom',
+        "top": "top",
+        "text-top": "top",
+        "middle": "center",
+        "baseline": "bottom",
+        "bottom": "bottom",
+        "text-bottom": "bottom",
         # OpenXML also has 'justify', 'distributed'
     }
 
-    def build_alignment(self, props):
+    def build_alignment(self, props) -> Dict[str, Optional[Union[bool, str]]]:
         # TODO: text-indent, padding-left -> alignment.indent
-        return {'horizontal': props.get('text-align'),
-                'vertical': self.VERTICAL_MAP.get(props.get('vertical-align')),
-                'wrap_text': (None if props.get('white-space') is None else
-                              props['white-space'] not in
-                              ('nowrap', 'pre', 'pre-line'))
-                }
+        return {
+            "horizontal": props.get("text-align"),
+            "vertical": self.VERTICAL_MAP.get(props.get("vertical-align")),
+            "wrap_text": (
+                None
+                if props.get("white-space") is None
+                else props["white-space"] not in ("nowrap", "pre", "pre-line")
+            ),
+        }
 
-    def build_border(self, props):
-        return {side: {
-            'style': self._border_style(props.get('border-{side}-style'
-                                        .format(side=side)),
-                                        props.get('border-{side}-width'
-                                        .format(side=side))),
-            'color': self.color_to_excel(
-                props.get('border-{side}-color'.format(side=side))),
-        } for side in ['top', 'right', 'bottom', 'left']}
+    def build_border(self, props: Dict) -> Dict[str, Dict[str, str]]:
+        return {
+            side: {
+                "style": self._border_style(
+                    props.get(f"border-{side}-style"),
+                    props.get(f"border-{side}-width"),
+                ),
+                "color": self.color_to_excel(props.get(f"border-{side}-color")),
+            }
+            for side in ["top", "right", "bottom", "left"]
+        }
 
-    def _border_style(self, style, width):
+    def _border_style(self, style: Optional[str], width):
         # convert styles and widths to openxml, one of:
         #       'dashDot'
         #       'dashDotDot'
@@ -153,61 +162,70 @@ class CSSToExcelConverter(object):
         #       'thin'
         if width is None and style is None:
             return None
-        if style == 'none' or style == 'hidden':
+        if style == "none" or style == "hidden":
             return None
 
         if width is None:
-            width = '2pt'
+            width = "2pt"
         width = float(width[:-2])
         if width < 1e-5:
             return None
         elif width < 1.3:
-            width_name = 'thin'
+            width_name = "thin"
         elif width < 2.8:
-            width_name = 'medium'
+            width_name = "medium"
         else:
-            width_name = 'thick'
+            width_name = "thick"
 
-        if style in (None, 'groove', 'ridge', 'inset', 'outset'):
+        if style in (None, "groove", "ridge", "inset", "outset"):
             # not handled
-            style = 'solid'
+            style = "solid"
 
-        if style == 'double':
-            return 'double'
-        if style == 'solid':
+        if style == "double":
+            return "double"
+        if style == "solid":
             return width_name
-        if style == 'dotted':
-            if width_name in ('hair', 'thin'):
-                return 'dotted'
-            return 'mediumDashDotDot'
-        if style == 'dashed':
-            if width_name in ('hair', 'thin'):
-                return 'dashed'
-            return 'mediumDashed'
+        if style == "dotted":
+            if width_name in ("hair", "thin"):
+                return "dotted"
+            return "mediumDashDotDot"
+        if style == "dashed":
+            if width_name in ("hair", "thin"):
+                return "dashed"
+            return "mediumDashed"
 
-    def build_fill(self, props):
+    def build_fill(self, props: Dict[str, str]):
         # TODO: perhaps allow for special properties
         #       -excel-pattern-bgcolor and -excel-pattern-type
-        fill_color = props.get('background-color')
-        if fill_color not in (None, 'transparent', 'none'):
-            return {
-                'fgColor': self.color_to_excel(fill_color),
-                'patternType': 'solid',
-            }
+        fill_color = props.get("background-color")
+        if fill_color not in (None, "transparent", "none"):
+            return {"fgColor": self.color_to_excel(fill_color), "patternType": "solid"}
 
-    BOLD_MAP = {'bold': True, 'bolder': True, '600': True, '700': True,
-                '800': True, '900': True,
-                'normal': False, 'lighter': False, '100': False, '200': False,
-                '300': False, '400': False, '500': False}
-    ITALIC_MAP = {'normal': False, 'italic': True, 'oblique': True}
+    BOLD_MAP = {
+        "bold": True,
+        "bolder": True,
+        "600": True,
+        "700": True,
+        "800": True,
+        "900": True,
+        "normal": False,
+        "lighter": False,
+        "100": False,
+        "200": False,
+        "300": False,
+        "400": False,
+        "500": False,
+    }
+    ITALIC_MAP = {"normal": False, "italic": True, "oblique": True}
 
-    def build_font(self, props):
-        size = props.get('font-size')
+    def build_font(self, props) -> Dict[str, Optional[Union[bool, int, str]]]:
+        size = props.get("font-size")
         if size is not None:
-            assert size.endswith('pt')
+            assert size.endswith("pt")
             size = float(size[:-2])
 
-        font_names_tmp = re.findall(r'''(?x)
+        font_names_tmp = re.findall(
+            r"""(?x)
             (
             "(?:[^"]|\\")+"
             |
@@ -215,13 +233,15 @@ class CSSToExcelConverter(object):
             |
             [^'",]+
             )(?=,|\s*$)
-        ''', props.get('font-family', ''))
+        """,
+            props.get("font-family", ""),
+        )
         font_names = []
         for name in font_names_tmp:
             if name[:1] == '"':
                 name = name[1:-1].replace('\\"', '"')
-            elif name[:1] == '\'':
-                name = name[1:-1].replace('\\\'', '\'')
+            elif name[:1] == "'":
+                name = name[1:-1].replace("\\'", "'")
             else:
                 name = name.strip()
             if name:
@@ -229,40 +249,41 @@ class CSSToExcelConverter(object):
 
         family = None
         for name in font_names:
-            if name == 'serif':
+            if name == "serif":
                 family = 1  # roman
                 break
-            elif name == 'sans-serif':
+            elif name == "sans-serif":
                 family = 2  # swiss
                 break
-            elif name == 'cursive':
+            elif name == "cursive":
                 family = 4  # script
                 break
-            elif name == 'fantasy':
+            elif name == "fantasy":
                 family = 5  # decorative
                 break
 
-        decoration = props.get('text-decoration')
+        decoration = props.get("text-decoration")
         if decoration is not None:
             decoration = decoration.split()
         else:
             decoration = ()
 
         return {
-            'name': font_names[0] if font_names else None,
-            'family': family,
-            'size': size,
-            'bold': self.BOLD_MAP.get(props.get('font-weight')),
-            'italic': self.ITALIC_MAP.get(props.get('font-style')),
-            'underline': ('single' if
-                          'underline' in decoration
-                          else None),
-            'strike': ('line-through' in decoration) or None,
-            'color': self.color_to_excel(props.get('color')),
+            "name": font_names[0] if font_names else None,
+            "family": family,
+            "size": size,
+            "bold": self.BOLD_MAP.get(props.get("font-weight")),
+            "italic": self.ITALIC_MAP.get(props.get("font-style")),
+            "underline": ("single" if "underline" in decoration else None),
+            "strike": ("line-through" in decoration) or None,
+            "color": self.color_to_excel(props.get("color")),
             # shadow if nonzero digit before shadow color
-            'shadow': (bool(re.search('^[^#(]*[1-9]',
-                                      props['text-shadow']))
-                       if 'text-shadow' in props else None),
+            "shadow": (
+                bool(re.search("^[^#(]*[1-9]", props["text-shadow"]))
+                if "text-shadow" in props
+                else None
+            ),
+            # FIXME: dont leave commented-out
             # 'vertAlign':,
             # 'charset': ,
             # 'scheme': ,
@@ -271,46 +292,45 @@ class CSSToExcelConverter(object):
         }
 
     NAMED_COLORS = {
-        'maroon': '800000',
-        'brown': 'A52A2A',
-        'red': 'FF0000',
-        'pink': 'FFC0CB',
-        'orange': 'FFA500',
-        'yellow': 'FFFF00',
-        'olive': '808000',
-        'green': '008000',
-        'purple': '800080',
-        'fuchsia': 'FF00FF',
-        'lime': '00FF00',
-        'teal': '008080',
-        'aqua': '00FFFF',
-        'blue': '0000FF',
-        'navy': '000080',
-        'black': '000000',
-        'gray': '808080',
-        'grey': '808080',
-        'silver': 'C0C0C0',
-        'white': 'FFFFFF',
+        "maroon": "800000",
+        "brown": "A52A2A",
+        "red": "FF0000",
+        "pink": "FFC0CB",
+        "orange": "FFA500",
+        "yellow": "FFFF00",
+        "olive": "808000",
+        "green": "008000",
+        "purple": "800080",
+        "fuchsia": "FF00FF",
+        "lime": "00FF00",
+        "teal": "008080",
+        "aqua": "00FFFF",
+        "blue": "0000FF",
+        "navy": "000080",
+        "black": "000000",
+        "gray": "808080",
+        "grey": "808080",
+        "silver": "C0C0C0",
+        "white": "FFFFFF",
     }
 
-    def color_to_excel(self, val):
+    def color_to_excel(self, val: Optional[str]):
         if val is None:
             return None
-        if val.startswith('#') and len(val) == 7:
+        if val.startswith("#") and len(val) == 7:
             return val[1:].upper()
-        if val.startswith('#') and len(val) == 4:
+        if val.startswith("#") and len(val) == 4:
             return (val[1] * 2 + val[2] * 2 + val[3] * 2).upper()
         try:
             return self.NAMED_COLORS[val]
         except KeyError:
-            warnings.warn('Unhandled color format: {val!r}'.format(val=val),
-                          CSSWarning)
+            warnings.warn(f"Unhandled color format: {repr(val)}", CSSWarning)
 
-    def build_number_format(self, props):
-        return {'format_code': props.get('number-format')}
+    def build_number_format(self, props: Dict) -> Dict[str, Optional[str]]:
+        return {"format_code": props.get("number-format")}
 
 
-class ExcelFormatter(object):
+class ExcelFormatter:
     """
     Class for formatting a DataFrame to a list of ExcelCells,
 
@@ -343,12 +363,25 @@ class ExcelFormatter(object):
         This is only called for body cells.
     """
 
-    def __init__(self, df, na_rep='', float_format=None, cols=None,
-                 header=True, index=True, index_label=None, merge_cells=False,
-                 inf_rep='inf', style_converter=None):
+    max_rows = 2 ** 20
+    max_cols = 2 ** 14
+
+    def __init__(
+        self,
+        df,
+        na_rep: str = "",
+        float_format: Optional[str] = None,
+        cols: Optional[Sequence] = None,
+        header: Union[bool, List[str]] = True,
+        index: bool = True,
+        index_label: Union[str, Sequence, None] = None,
+        merge_cells: bool = False,
+        inf_rep: str = "inf",
+        style_converter: Optional[Callable] = None,
+    ):
         self.rowcounter = 0
         self.na_rep = na_rep
-        if hasattr(df, 'render'):
+        if hasattr(df, "render"):
             self.styler = df
             df = df.data
             if style_converter is None:
@@ -361,18 +394,14 @@ class ExcelFormatter(object):
 
             # all missing, raise
             if not len(Index(cols) & df.columns):
-                raise KeyError(
-                    "passes columns are not ALL present dataframe")
+                raise KeyError("passes columns are not ALL present dataframe")
 
-            # deprecatedin gh-17295
-            # 1 missing is ok (for now)
             if len(Index(cols) & df.columns) != len(cols):
-                warnings.warn(
-                    "Not all names specified in 'columns' are found; "
-                    "this will raise a KeyError in the future",
-                    FutureWarning)
+                # Deprecated in GH#17295, enforced in 1.0.0
+                raise KeyError("Not all names specified in 'columns' are found")
 
             self.df = df.reindex(columns=cols)
+
         self.columns = self.df.columns
         self.float_format = float_format
         self.index = index
@@ -383,13 +412,16 @@ class ExcelFormatter(object):
 
     @property
     def header_style(self):
-        return {"font": {"bold": True},
-                "borders": {"top": "thin",
-                            "right": "thin",
-                            "bottom": "thin",
-                            "left": "thin"},
-                "alignment": {"horizontal": "center",
-                              "vertical": "top"}}
+        return {
+            "font": {"bold": True},
+            "borders": {
+                "top": "thin",
+                "right": "thin",
+                "bottom": "thin",
+                "left": "thin",
+            },
+            "alignment": {"horizontal": "center", "vertical": "top"},
+        }
 
     def _format_value(self, val):
         if is_scalar(val) and missing.isna(val):
@@ -398,26 +430,33 @@ class ExcelFormatter(object):
             if missing.isposinf_scalar(val):
                 val = self.inf_rep
             elif missing.isneginf_scalar(val):
-                val = '-{inf}'.format(inf=self.inf_rep)
+                val = f"-{self.inf_rep}"
             elif self.float_format is not None:
                 val = float(self.float_format % val)
+        if getattr(val, "tzinfo", None) is not None:
+            raise ValueError(
+                "Excel does not support datetimes with "
+                "timezones. Please ensure that datetimes "
+                "are timezone unaware before writing to Excel."
+            )
         return val
 
     def _format_header_mi(self):
         if self.columns.nlevels > 1:
             if not self.index:
-                raise NotImplementedError("Writing to Excel with MultiIndex"
-                                          " columns and no index "
-                                          "('index'=False) is not yet "
-                                          "implemented.")
+                raise NotImplementedError(
+                    "Writing to Excel with MultiIndex columns and no "
+                    "index ('index'=False) is not yet implemented."
+                )
 
         has_aliases = isinstance(self.header, (tuple, list, np.ndarray, Index))
         if not (has_aliases or self.header):
             return
 
         columns = self.columns
-        level_strs = columns.format(sparsify=self.merge_cells, adjoin=False,
-                                    names=False)
+        level_strs = columns.format(
+            sparsify=self.merge_cells, adjoin=False, names=False
+        )
         level_lengths = get_level_lengths(level_strs)
         coloffset = 0
         lnum = 0
@@ -431,17 +470,24 @@ class ExcelFormatter(object):
                 name = columns.names[lnum]
                 yield ExcelCell(lnum, coloffset, name, self.header_style)
 
-            for lnum, (spans, levels, level_codes) in enumerate(zip(
-                    level_lengths, columns.levels, columns.codes)):
+            for lnum, (spans, levels, level_codes) in enumerate(
+                zip(level_lengths, columns.levels, columns.codes)
+            ):
                 values = levels.take(level_codes)
                 for i in spans:
                     if spans[i] > 1:
-                        yield ExcelCell(lnum, coloffset + i + 1, values[i],
-                                        self.header_style, lnum,
-                                        coloffset + i + spans[i])
+                        yield ExcelCell(
+                            lnum,
+                            coloffset + i + 1,
+                            values[i],
+                            self.header_style,
+                            lnum,
+                            coloffset + i + spans[i],
+                        )
                     else:
-                        yield ExcelCell(lnum, coloffset + i + 1, values[i],
-                                        self.header_style)
+                        yield ExcelCell(
+                            lnum, coloffset + i + 1, values[i], self.header_style
+                        )
         else:
             # Format in legacy format with dots to indicate levels.
             for i, values in enumerate(zip(*level_strs)):
@@ -463,15 +509,17 @@ class ExcelFormatter(object):
             colnames = self.columns
             if has_aliases:
                 if len(self.header) != len(self.columns):
-                    raise ValueError('Writing {cols} cols but got {alias} '
-                                     'aliases'.format(cols=len(self.columns),
-                                                      alias=len(self.header)))
+                    raise ValueError(
+                        f"Writing {len(self.columns)} cols but got {len(self.header)} "
+                        "aliases"
+                    )
                 else:
                     colnames = self.header
 
             for colindex, colname in enumerate(colnames):
-                yield ExcelCell(self.rowcounter, colindex + coloffset, colname,
-                                self.header_style)
+                yield ExcelCell(
+                    self.rowcounter, colindex + coloffset, colname, self.header_style
+                )
 
     def _format_header(self):
         if isinstance(self.columns, ABCMultiIndex):
@@ -481,17 +529,18 @@ class ExcelFormatter(object):
 
         gen2 = ()
         if self.df.index.names:
-            row = [x if x is not None else ''
-                   for x in self.df.index.names] + [''] * len(self.columns)
-            if reduce(lambda x, y: x and y, map(lambda x: x != '', row)):
-                gen2 = (ExcelCell(self.rowcounter, colindex, val,
-                                  self.header_style)
-                        for colindex, val in enumerate(row))
+            row = [x if x is not None else "" for x in self.df.index.names] + [
+                ""
+            ] * len(self.columns)
+            if reduce(lambda x, y: x and y, map(lambda x: x != "", row)):
+                gen2 = (
+                    ExcelCell(self.rowcounter, colindex, val, self.header_style)
+                    for colindex, val in enumerate(row)
+                )
                 self.rowcounter += 1
         return itertools.chain(gen, gen2)
 
     def _format_body(self):
-
         if isinstance(self.df.index, ABCMultiIndex):
             return self._format_hierarchical_rows()
         else:
@@ -506,9 +555,9 @@ class ExcelFormatter(object):
         if self.index:
             # check aliases
             # if list only take first as this is not a MultiIndex
-            if (self.index_label and
-                    isinstance(self.index_label, (list, tuple, np.ndarray,
-                                                  Index))):
+            if self.index_label and isinstance(
+                self.index_label, (list, tuple, np.ndarray, Index)
+            ):
                 index_label = self.index_label[0]
             # if string good to go
             elif self.index_label and isinstance(self.index_label, str):
@@ -520,8 +569,7 @@ class ExcelFormatter(object):
                 self.rowcounter += 1
 
             if index_label and self.header is not False:
-                yield ExcelCell(self.rowcounter - 1, 0, index_label,
-                                self.header_style)
+                yield ExcelCell(self.rowcounter - 1, 0, index_label, self.header_style)
 
             # write index_values
             index_values = self.df.index
@@ -529,8 +577,7 @@ class ExcelFormatter(object):
                 index_values = self.df.index.to_timestamp()
 
             for idx, idxval in enumerate(index_values):
-                yield ExcelCell(self.rowcounter + idx, 0, idxval,
-                                self.header_style)
+                yield ExcelCell(self.rowcounter + idx, 0, idxval, self.header_style)
 
             coloffset = 1
         else:
@@ -549,62 +596,74 @@ class ExcelFormatter(object):
         if self.index:
             index_labels = self.df.index.names
             # check for aliases
-            if (self.index_label and
-                    isinstance(self.index_label, (list, tuple, np.ndarray,
-                                                  Index))):
+            if self.index_label and isinstance(
+                self.index_label, (list, tuple, np.ndarray, Index)
+            ):
                 index_labels = self.index_label
 
             # MultiIndex columns require an extra row
             # with index names (blank if None) for
-            # unambigous round-trip, unless not merging,
+            # unambiguous round-trip, unless not merging,
             # in which case the names all go on one row Issue #11328
             if isinstance(self.columns, ABCMultiIndex) and self.merge_cells:
                 self.rowcounter += 1
 
             # if index labels are not empty go ahead and dump
-            if com._any_not_none(*index_labels) and self.header is not False:
+            if com.any_not_none(*index_labels) and self.header is not False:
 
                 for cidx, name in enumerate(index_labels):
-                    yield ExcelCell(self.rowcounter - 1, cidx, name,
-                                    self.header_style)
+                    yield ExcelCell(self.rowcounter - 1, cidx, name, self.header_style)
 
             if self.merge_cells:
                 # Format hierarchical rows as merged cells.
-                level_strs = self.df.index.format(sparsify=True, adjoin=False,
-                                                  names=False)
+                level_strs = self.df.index.format(
+                    sparsify=True, adjoin=False, names=False
+                )
                 level_lengths = get_level_lengths(level_strs)
 
-                for spans, levels, level_codes in zip(level_lengths,
-                                                      self.df.index.levels,
-                                                      self.df.index.codes):
+                for spans, levels, level_codes in zip(
+                    level_lengths, self.df.index.levels, self.df.index.codes
+                ):
 
-                    values = levels.take(level_codes,
-                                         allow_fill=levels._can_hold_na,
-                                         fill_value=True)
+                    values = levels.take(
+                        level_codes, allow_fill=levels._can_hold_na, fill_value=True
+                    )
 
                     for i in spans:
                         if spans[i] > 1:
-                            yield ExcelCell(self.rowcounter + i, gcolidx,
-                                            values[i], self.header_style,
-                                            self.rowcounter + i + spans[i] - 1,
-                                            gcolidx)
+                            yield ExcelCell(
+                                self.rowcounter + i,
+                                gcolidx,
+                                values[i],
+                                self.header_style,
+                                self.rowcounter + i + spans[i] - 1,
+                                gcolidx,
+                            )
                         else:
-                            yield ExcelCell(self.rowcounter + i, gcolidx,
-                                            values[i], self.header_style)
+                            yield ExcelCell(
+                                self.rowcounter + i,
+                                gcolidx,
+                                values[i],
+                                self.header_style,
+                            )
                     gcolidx += 1
 
             else:
                 # Format hierarchical rows with non-merged values.
                 for indexcolvals in zip(*self.df.index):
                     for idx, indexcolval in enumerate(indexcolvals):
-                        yield ExcelCell(self.rowcounter + idx, gcolidx,
-                                        indexcolval, self.header_style)
+                        yield ExcelCell(
+                            self.rowcounter + idx,
+                            gcolidx,
+                            indexcolval,
+                            self.header_style,
+                        )
                     gcolidx += 1
 
         for cell in self._generate_body(gcolidx):
             yield cell
 
-    def _generate_body(self, coloffset):
+    def _generate_body(self, coloffset: int):
         if self.styler is None:
             styles = None
         else:
@@ -618,18 +677,23 @@ class ExcelFormatter(object):
             series = self.df.iloc[:, colidx]
             for i, val in enumerate(series):
                 if styles is not None:
-                    xlstyle = self.style_converter(';'.join(styles[i, colidx]))
-                yield ExcelCell(self.rowcounter + i, colidx + coloffset, val,
-                                xlstyle)
+                    xlstyle = self.style_converter(";".join(styles[i, colidx]))
+                yield ExcelCell(self.rowcounter + i, colidx + coloffset, val, xlstyle)
 
     def get_formatted_cells(self):
-        for cell in itertools.chain(self._format_header(),
-                                    self._format_body()):
+        for cell in itertools.chain(self._format_header(), self._format_body()):
             cell.val = self._format_value(cell.val)
             yield cell
 
-    def write(self, writer, sheet_name='Sheet1', startrow=0,
-              startcol=0, freeze_panes=None, engine=None):
+    def write(
+        self,
+        writer,
+        sheet_name="Sheet1",
+        startrow=0,
+        startcol=0,
+        freeze_panes=None,
+        engine=None,
+    ):
         """
         writer : string or ExcelWriter object
             File path or existing ExcelWriter
@@ -648,17 +712,27 @@ class ExcelFormatter(object):
             and ``io.excel.xlsm.writer``.
         """
         from pandas.io.excel import ExcelWriter
-        from pandas.io.common import _stringify_path
+
+        num_rows, num_cols = self.df.shape
+        if num_rows > self.max_rows or num_cols > self.max_cols:
+            raise ValueError(
+                f"This sheet is too large! Your sheet size is: {num_rows}, {num_cols} "
+                f"Max sheet size is: {self.max_rows}, {self.max_cols}"
+            )
 
         if isinstance(writer, ExcelWriter):
             need_save = False
         else:
-            writer = ExcelWriter(_stringify_path(writer), engine=engine)
+            writer = ExcelWriter(stringify_path(writer), engine=engine)
             need_save = True
 
         formatted_cells = self.get_formatted_cells()
-        writer.write_cells(formatted_cells, sheet_name,
-                           startrow=startrow, startcol=startcol,
-                           freeze_panes=freeze_panes)
+        writer.write_cells(
+            formatted_cells,
+            sheet_name,
+            startrow=startrow,
+            startcol=startcol,
+            freeze_panes=freeze_panes,
+        )
         if need_save:
             writer.save()
