@@ -1,7 +1,6 @@
 import pytest
 
-from pandas import Series
-import pandas.util.testing as tm
+import pandas._testing as tm
 
 
 @pytest.fixture
@@ -10,7 +9,7 @@ def datetime_series():
     Fixture for Series of floats with DatetimeIndex
     """
     s = tm.makeTimeSeries()
-    s.name = 'ts'
+    s.name = "ts"
     return s
 
 
@@ -20,23 +19,15 @@ def string_series():
     Fixture for Series of floats with Index of unique strings
     """
     s = tm.makeStringSeries()
-    s.name = 'series'
+    s.name = "series"
     return s
 
 
 @pytest.fixture
 def object_series():
     """
-    Fixture for Series of dtype datetime64[ns] with Index of unique strings
+    Fixture for Series of dtype object with Index of unique strings
     """
     s = tm.makeObjectSeries()
-    s.name = 'objects'
+    s.name = "objects"
     return s
-
-
-@pytest.fixture
-def empty_series():
-    """
-    Fixture for empty Series
-    """
-    return Series([], index=[])
